@@ -3,17 +3,21 @@ use super::context::Context;
 use super::token_finder::TokenFinder;
 use dprint_core::formatting::*;
 use jsonc_parser::ast::*;
-use jsonc_parser::common::{Position, Range, Ranged};
+use jsonc_parser::common::Range;
+use jsonc_parser::common::Ranged;
 use jsonc_parser::tokens::TokenAndRange;
 use std::collections::HashSet;
+use text_lines::TextLines;
 
 pub fn generate(parse_result: jsonc_parser::ParseResult, text: &str, config: &Configuration) -> PrintItems {
   let comments = parse_result.comments.unwrap();
   let tokens = parse_result.tokens.unwrap();
   let node_value = parse_result.value;
+  let text_info = TextLines::new(text);
   let mut context = Context {
     config,
     text,
+    text_info,
     handled_comments: HashSet::new(),
     parent_stack: Vec::new(),
     current_node: None,
@@ -25,10 +29,8 @@ pub fn generate(parse_result: jsonc_parser::ParseResult, text: &str, config: &Co
   if let Some(node_value) = &node_value {
     items.extend(gen_node(node_value.into(), &mut context));
     items.extend(gen_trailing_comments_as_statements(node_value, &mut context));
-  } else {
-    if let Some(comments) = comments.get(&0) {
-      items.extend(gen_comments_as_statements(comments.iter(), None, &mut context));
-    }
+  } else if let Some(comments) = comments.get(&0) {
+    items.extend(gen_comments_as_statements(comments.iter(), None, &mut context));
   }
   items.push_condition(conditions::if_true(
     "endOfFileNewLine",
@@ -102,7 +104,13 @@ fn gen_node_with_inner<'a>(
 
 fn gen_array<'a>(node: &'a Array<'a>, context: &mut Context<'a, '_>) -> PrintItems {
   let force_multi_lines = !context.config.array_prefer_single_line
-    && (should_break_up_single_line(node, context) || node.range.start_line < node.elements.first().map(|p| p.start_line()).unwrap_or(node.range.start_line));
+    && (should_break_up_single_line(node, context)
+      || context.text_info.line_index(node.start())
+        < node
+          .elements
+          .first()
+          .map(|p| context.text_info.line_index(p.start()))
+          .unwrap_or_else(|| context.text_info.line_index(node.start())));
 
   gen_surrounded_by_tokens(
     |context| {
@@ -126,7 +134,7 @@ fn gen_array<'a>(node: &'a Array<'a>, context: &mut Context<'a, '_>) -> PrintIte
     GenSurroundedByTokensOptions {
       open_token: "[",
       close_token: "]",
-      range: node.range.clone(),
+      range: node.range,
       first_member: node.elements.first().map(|f| f.range()),
       prefer_single_line_when_empty: true,
     },
@@ -136,7 +144,13 @@ fn gen_array<'a>(node: &'a Array<'a>, context: &mut Context<'a, '_>) -> PrintIte
 
 fn gen_object<'a>(obj: &'a Object, context: &mut Context<'a, '_>) -> PrintItems {
   let force_multi_lines = !context.config.object_prefer_single_line
-    && (should_break_up_single_line(obj, context) || obj.range.start_line < obj.properties.first().map(|p| p.range.start_line).unwrap_or(obj.range.end_line));
+    && (should_break_up_single_line(obj, context)
+      || context.text_info.line_index(obj.start())
+        < obj
+          .properties
+          .first()
+          .map(|p| context.text_info.line_index(p.start()))
+          .unwrap_or_else(|| context.text_info.line_index(obj.end())));
 
   gen_surrounded_by_tokens(
     |context| {
@@ -160,7 +174,7 @@ fn gen_object<'a>(obj: &'a Object, context: &mut Context<'a, '_>) -> PrintItems
     GenSurroundedByTokensOptions {
       open_token: "{",
       close_token: "}",
-      range: obj.range.clone(),
+      range: obj.range,
       first_member: obj.properties.first().map(|f| &f.range),
       prefer_single_line_when_empty: false,
     },
@@ -179,7 +193,7 @@ fn gen_object_prop<'a>(node: &'a ObjectProp, context: &mut Context<'a, '_>) -> P
 
 fn gen_string_lit<'a>(node: &'a StringLit, context: &mut Context<'a, '_>) -> PrintItems {
   let text = node.text(context.text);
-  let is_double_quotes = text.chars().next().unwrap() == '"';
+  let is_double_quotes = text.starts_with('"');
   let mut items = PrintItems::new();
   let text = &text[1..text.len() - 1];
   let text = if is_double_quotes {
@@ -256,7 +270,7 @@ fn gen_comma_separated_values<'a>(opts: GenCommaSeparatedValuesOptions<'a>, cont
       allow_blank_lines: opts.allow_blank_lines,
       single_line_space_at_start: opts.single_line_space_at_start,
       single_line_space_at_end: opts.single_line_space_at_end,
-      single_line_separator: opts.custom_single_line_separator.unwrap_or(Signal::SpaceOrNewLine.into()),
+      single_line_separator: opts.custom_single_line_separator.unwrap_or_else(|| Signal::SpaceOrNewLine.into()),
       indent_width,
       multi_line_options: opts.multi_line_options,
       force_possible_newline_at_start: opts.force_possible_newline_at_start,
@@ -273,7 +287,7 @@ fn gen_comma_separated_value<'a>(value: Option<Node<'a, 'a>>, generated_comma: P
     let generated_comma = generated_comma.into_rc_path();
     items.extend(gen_node_with_inner(element, context, move |mut items, _| {
       // this Rc clone is necessary because we can't move the captured generated_comma out of this closure
-      items.push_optional_path(generated_comma.clone());
+      items.push_optional_path(generated_comma);
       items
     }));
   } else {
@@ -309,24 +323,24 @@ fn gen_surrounded_by_tokens<'a, 'b>(
   opts: GenSurroundedByTokensOptions<'a>,
   context: &mut Context<'a, 'b>,
 ) -> PrintItems {
-  let open_token_end = Position::new(opts.range.start + opts.open_token.len(), opts.range.start_line);
-  let close_token_start = Position::new(opts.range.end - opts.close_token.len(), opts.range.end_line);
+  let open_token_end = opts.range.start + opts.open_token.len();
+  let close_token_start = opts.range.end - opts.close_token.len();
 
   // assert the tokens are in the place the caller says they are
   #[cfg(debug_assertions)]
-  context.assert_text(opts.range.start, open_token_end.range.end, opts.open_token);
+  context.assert_text(opts.range.start, open_token_end, opts.open_token);
   #[cfg(debug_assertions)]
-  context.assert_text(close_token_start.range.start, opts.range.end, opts.close_token);
+  context.assert_text(close_token_start, opts.range.end, opts.close_token);
 
   // generate
   let mut items = PrintItems::new();
-  let open_token_start_line = opts.range.start_line;
+  let open_token_start_line = context.text_info.line_index(opts.range.start);
 
   items.push_str(opts.open_token);
   if let Some(first_member) = opts.first_member {
-    let first_member_start_line = first_member.start_line;
+    let first_member_start_line = context.text_info.line_index(first_member.start);
     if open_token_start_line < first_member_start_line {
-      if let Some(trailing_comments) = context.comments.get(&open_token_end.start()) {
+      if let Some(trailing_comments) = context.comments.get(&open_token_end) {
         items.extend(gen_first_line_trailing_comment(open_token_start_line, trailing_comments.iter(), context));
       }
     }
@@ -334,21 +348,25 @@ fn gen_surrounded_by_tokens<'a, 'b>(
 
     let before_trailing_comments_info = Info::new("beforeTrailingComments");
     items.push_info(before_trailing_comments_info);
-    items.extend(ir_helpers::with_indent(gen_trailing_comments_as_statements(&open_token_end, context)));
-    if let Some(leading_comments) = context.comments.get(&close_token_start.start()) {
+    items.extend(ir_helpers::with_indent(gen_trailing_comments_as_statements(
+      &Range::from_byte_index(open_token_end),
+      context,
+    )));
+    if let Some(leading_comments) = context.comments.get(&close_token_start) {
       items.extend(ir_helpers::with_indent(gen_comments_as_statements(leading_comments.iter(), None, context)));
     }
     items.push_condition(conditions::if_true(
       "newLineIfHasCommentsAndNotStartOfNewLine",
       move |context| {
         let had_comments = !condition_resolvers::is_at_same_position(context, &before_trailing_comments_info)?;
-        return Some(had_comments && !context.writer_info.is_start_of_line());
+        Some(had_comments && !context.writer_info.is_start_of_line())
       },
       Signal::NewLine.into(),
     ));
   } else {
-    let is_single_line = open_token_start_line == opts.range.end_line;
-    if let Some(comments) = context.comments.get(&open_token_end.start()) {
+    let range_end_line = context.text_info.line_index(opts.range.end);
+    let is_single_line = open_token_start_line == range_end_line;
+    if let Some(comments) = context.comments.get(&open_token_end) {
       // generate the trailing comment on the first line only if multi-line and if a comment line
       if !is_single_line {
         items.extend(gen_first_line_trailing_comment(open_token_start_line, comments.iter(), context));
@@ -363,8 +381,8 @@ fn gen_surrounded_by_tokens<'a, 'b>(
               |_| {
                 let mut generated_comments = Vec::new();
                 for c in comments.iter() {
-                  let start_line = c.start_line();
-                  let end_line = c.end_line();
+                  let start_line = context.text_info.line_index(c.start());
+                  let end_line = context.text_info.line_index(c.end());
                   if let Some(items) = gen_comment(c, context) {
                     generated_comments.push(ir_helpers::GeneratedValue {
                       items,
@@ -396,10 +414,8 @@ fn gen_surrounded_by_tokens<'a, 'b>(
           items.push_signal(Signal::NewLine);
         }
       }
-    } else {
-      if !is_single_line && !opts.prefer_single_line_when_empty {
-        items.push_signal(Signal::NewLine);
-      }
+    } else if !is_single_line && !opts.prefer_single_line_when_empty {
+      items.push_signal(Signal::NewLine);
     }
   }
 
@@ -415,8 +431,8 @@ fn gen_surrounded_by_tokens<'a, 'b>(
     let mut items = PrintItems::new();
     let mut comments = comments;
     if let Some(first_comment) = comments.next() {
-      if first_comment.kind() == CommentKind::Line && first_comment.start_line() == open_token_start_line {
-        if let Some(generated_comment) = gen_comment(&first_comment, context) {
+      if first_comment.kind() == CommentKind::Line && context.text_info.line_index(first_comment.start()) == open_token_start_line {
+        if let Some(generated_comment) = gen_comment(first_comment, context) {
           items.push_signal(Signal::StartForceNoNewLines);
           items.push_str(" ");
           items.extend(generated_comment);
@@ -430,8 +446,8 @@ fn gen_surrounded_by_tokens<'a, 'b>(
 
 // Comments
 
-fn has_unhandled_comment<'a: 'b, 'b>(comments: impl Iterator<Item = &'b Comment<'a>>, context: &mut Context) -> bool {
-  comments.filter(|c| !context.has_handled_comment(c)).next().is_some()
+fn has_unhandled_comment<'a: 'b, 'b>(mut comments: impl Iterator<Item = &'b Comment<'a>>, context: &mut Context) -> bool {
+  comments.any(|c| !context.has_handled_comment(c))
 }
 
 fn gen_trailing_comments(node: &dyn Ranged, context: &mut Context) -> PrintItems {
@@ -449,10 +465,10 @@ fn gen_trailing_comments_as_statements(node: &dyn Ranged, context: &mut Context)
 
 fn get_trailing_comments_as_statements<'a, 'b>(node: &dyn Ranged, context: &mut Context<'a, 'b>) -> Vec<&'b Comment<'a>> {
   let mut comments = Vec::new();
-  let node_end_line = node.end_line();
+  let node_end_line = context.text_info.line_index(node.end());
   if let Some(trailing_comments) = context.comments.get(&node.end()) {
     for comment in trailing_comments.iter() {
-      if !context.has_handled_comment(comment) && node_end_line < comment.end_line() {
+      if !context.has_handled_comment(comment) && node_end_line < context.text_info.line_index(comment.end()) {
         comments.push(comment);
       }
     }
@@ -487,11 +503,11 @@ fn gen_comments_as_leading<'a: 'b, 'b>(node: &dyn Ranged, comments: impl Iterato
 
   if !comments.is_empty() {
     let last_comment = comments.last().unwrap();
-    let last_comment_end_line = last_comment.end_line();
+    let last_comment_end_line = context.text_info.line_index(last_comment.end());
     let last_comment_kind = last_comment.kind();
     items.extend(gen_comment_collection(comments.into_iter(), None, Some(node), context));
 
-    let node_start_line = node.start_line();
+    let node_start_line = context.text_info.line_index(node.start());
     if node_start_line > last_comment_end_line {
       items.push_signal(Signal::NewLine);
 
@@ -508,10 +524,12 @@ fn gen_comments_as_leading<'a: 'b, 'b>(node: &dyn Ranged, comments: impl Iterato
 
 fn gen_comments_as_trailing<'a: 'b, 'b>(node: &dyn Ranged, comments: impl Iterator<Item = &'b Comment<'a>>, context: &mut Context) -> PrintItems {
   // use the roslyn definition of trailing comments
-  let node_end_line = node.end_line();
-  let trailing_comments_on_same_line = comments.filter(|c| c.start_line() <= node_end_line).collect::<Vec<_>>();
+  let node_end_line = context.text_info.line_index(node.end());
+  let trailing_comments_on_same_line = comments
+    .filter(|c| context.text_info.line_index(c.start()) <= node_end_line)
+    .collect::<Vec<_>>();
 
-  let first_unhandled_comment = trailing_comments_on_same_line.iter().filter(|c| !context.has_handled_comment(c)).next();
+  let first_unhandled_comment = trailing_comments_on_same_line.iter().find(|c| !context.has_handled_comment(c));
   let mut items = PrintItems::new();
 
   if let Some(Comment::Block(_)) = first_unhandled_comment {
@@ -531,7 +549,7 @@ fn gen_comment_collection<'a: 'b, 'b>(
 ) -> PrintItems {
   let mut last_node = last_node;
   let mut items = PrintItems::new();
-  let next_node_start_line = next_node.map(|n| n.start_line());
+  let next_node_start_line = next_node.map(|n| context.text_info.line_index(n.start()));
 
   for comment in comments {
     if !context.has_handled_comment(comment) {
@@ -540,7 +558,7 @@ fn gen_comment_collection<'a: 'b, 'b>(
         &last_node,
         GenCommentBasedOnLastNodeOptions {
           separate_with_newlines: if let Some(next_node_start_line) = next_node_start_line {
-            comment.start_line() != next_node_start_line
+            context.text_info.line_index(comment.start()) != next_node_start_line
           } else {
             false
           },
@@ -568,8 +586,8 @@ fn gen_comment_based_on_last_node(
   let mut pushed_ignore_new_lines = false;
 
   if let Some(last_node) = last_node {
-    let comment_start_line = comment.start_line();
-    let last_node_end_line = last_node.end_line();
+    let comment_start_line = context.text_info.line_index(comment.start());
+    let last_node_end_line = context.text_info.line_index(last_node.end());
 
     if opts.separate_with_newlines || comment_start_line > last_node_end_line {
       items.push_signal(Signal::NewLine);
@@ -586,7 +604,7 @@ fn gen_comment_based_on_last_node(
     }
   }
 
-  if let Some(generated_comment) = gen_comment(&comment, context) {
+  if let Some(generated_comment) = gen_comment(comment, context) {
     items.extend(generated_comment);
   }
 
@@ -594,7 +612,7 @@ fn gen_comment_based_on_last_node(
     items.push_signal(Signal::FinishForceNoNewLines);
   }
 
-  return items;
+  items
 }
 
 fn gen_comment(comment: &Comment, context: &mut Context) -> Option<PrintItems> {
@@ -606,8 +624,8 @@ fn gen_comment(comment: &Comment, context: &mut Context) -> Option<PrintItems> {
   // mark handled and generate
   context.mark_comment_handled(comment);
   Some(match comment {
-    Comment::Block(comment) => ir_helpers::gen_js_like_comment_block(&comment.text),
-    Comment::Line(comment) => ir_helpers::gen_js_like_comment_line(&comment.text, context.config.comment_line_force_space_after_slashes),
+    Comment::Block(comment) => ir_helpers::gen_js_like_comment_block(comment.text),
+    Comment::Line(comment) => ir_helpers::gen_js_like_comment_line(comment.text, context.config.comment_line_force_space_after_slashes),
   })
 }
 
@@ -628,5 +646,5 @@ fn should_break_up_single_line(ranged: &impl Ranged, context: &Context) -> bool
   // Obviously this line_width * 2 is not always accurate as it doesn't take into account whitespace,
   // but will provide a good enough and fast way to quickly tell if it's long without having basically
   // any false positives (unless someone is being silly).
-  range.start_line == range.end_line && range.width() > (context.config.line_width * 2) as usize
+  context.text_info.line_index(range.start) == context.text_info.line_index(range.end) && range.width() > (context.config.line_width * 2) as usize
 }
